/*******************************************************************************

  TEMPORIZADOR PROGRAMAVEL PARA ILUMINAO NOTURNA

  C/ PIC 12F675

  AUTOR: CLUDIO LRIOS

  DATA: 19/10/2016

  APLICAO: LIGAR DIARIAMENTE E DE FORMA AUTOMATICA ILUMINAO NO HORRIO E
  PELO PERIODO DE TEMPO PROGRAMADO PELO USURIO.

  USO: O PERIODO LIGADO PODE SER PROGRAMADO P/ 2,4,6,8,10 OU 12 HORAS, POR MEIO
  DE UM JUMPER. CONFORME A POSIO QUE  COLOCADO TEREMOS UM PERIODO DIFERENTE.
  O HORRIO EM QUE SE ATIVAR A SADA DIARIAMENTE SER O MESMO HORRIO DE QUANDO
  FOR APERTADO A TECLA 'START'.

  EX. PROGRAMAMOS JUMPER P/ 4 HS.
  AS 19 HORAS APERTAMOS O BOTO 'START'. LIGAR A   SADA AT AS 23 HORAS E
  ENTO DESLIGAR.
  NOS DIAS A FRENTE, AS 19 HORAS LIGAR  NOVAMENTE AT AS 23 HORAS, EM UM CICLO
  CONTNUO, AT QUE SEJA ALTERADO A PROGRAMAO.
  PARA QUEDAS MOMENTANEAS DE ENERGIA ELTRICA DA REDE, FOI PREVISTO O USO DE UM
  SUPERCAPACITOR DE .47 FARADS (470.000 uF).
  COMO OPO, UMA BATERIA DE 3,3 VOLTS PODER SER USADA, DESDE QUE SEJA COLOCADO
  UM DIODO EM SRIE, PARA A TENSO DA FONTE NO DANIFICA-LA.
  USE DIODO COM TENSO DE BARREIRA PEQUENA (0,2 VOLTS).

  AO LIGAR PELA PRIMEIRA VEZ, OU SE HOUVER QUEDA DE ENERGIA SEM USO DE SUPERCAPA-
  CITOR OU BATERIA, O LED 'PROG' IR   PISCAR RAPIDAMENTE INDICANDO QUE PRECISA
  SER REPROGRAMADO.
  APS A PROGRAMAO (POSICIONANDO O JUMPER E APERTANDO O BOTO 'START') TEREMOS
  INFORMAO VISUAL DESTA PROGRAMAO CONTANDO QUANTAS PISCADAS OCORREM ENTRE OS
  INTERVALOS SEM  PISCAR. CADA PISCADA EQUIVALE A 1 HORA DE PERIODO LIGADO.



  APLICAO DIDTICA

  ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=32768)
#fuses NOWDT,LP, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT

//bytes
#byte TMR0 = 0x01
#byte TRISIO = 0x85
#byte OPTION_REG = 0x81
#byte ANSEL = 0x9f
#byte CMCON = 0x19

//bits
#bit  SW  = 0x5.3  //boto de programao 'start'
#bit  OUT = 0x5.0 //sada para controlar a lampada
#bit  LED = 0x5.1 //led indicador de programao
#bit t0if = 0xb.2 //flag de over flow do timer0

#define LED_LIGADO 0
#define LED_DESLIGADO 1

#define SAIDA_LIGADA 0
#define SAIDA_DESLIGADA 1

#define PAUSE_PISCA 3 // intervalo entre piscada informativa

//variavis globais
//8bits
unsigned int8 cont1=0;//contador para obteno de piscadas no LED PROG
unsigned int8 seg=0;//armazena 'segundos'
unsigned int8 min=0;//armazena 'minutos'
unsigned int8 hora=0;//armazena 'horas'
unsigned int8 periodo=2;//armazena o periodo que ficar ligado
unsigned int8 vjump=0;// auxiliar para teste do jumper
unsigned int8 n_piscada=0;//auxiliar para obter piscadas informativas
unsigned int8 t_pisc=PAUSE_PISCA; // idem
//flags
unsigned int1 f_prog=1;//indica programado (0)/no programado (1)
unsigned int1 f_pisca=1;//indica que LED PROG deve piscar rpidamente
unsigned int1 f_disp=0;//indica que chegou a hora de ligar a sada
unsigned int1 f_t1=0;//trava auxiliar do boto 'start'


/*******************************************************************************
               FUNES GERAIS
*******************************************************************************/
void Atualiza_Cont_Tempo()
{
   seg++;
    if ( seg >= 60) {
       seg = 0;
       min++;
       if ( min >= 60) {
          min = 0;
          hora++;
          if ( hora >= 24){
            hora = 0;
            f_disp=1;//dispara processo com inicio do tempo programado
          }
       }
    }
 }
/*******************************************************************************
               PISCA LED PROG RAPIDAMENTE
*******************************************************************************/
void Pisca_Led() 
  #define vel_pisc 50 // frequncia da piscada 
 {
   if(f_pisca){
   cont1++;
   if(cont1>vel_pisc){cont1=0; LED=!LED;}
   }

   else LED=LED_DESLIGADO;//desliga led
 }

/*******************************************************************************
               DELAY PARA PISCADAS INFORMATIVAS
*******************************************************************************/
 void Delay_Pisca_ON(){
   unsigned int8 a;
    #define delay_on 100
    for(a=0;a<delay_on;a++);
 }

/*******************************************************************************
               LE A TENSO RESULTANTE DA ESCOLHA DO JUMPER
*******************************************************************************/
  void Le_Jumper(){
  unsigned int8 a;
  unsigned int16 val;

  val=0;
  for(a=0;a<8;a++){
    val+=READ_ADC();// l valor da tenso de entrada do pino 5 e salva
  }
  vjump=val/8;//realiza a mdia das 8 medies
  if(vjump>229)periodo=12;//valor para 12 horas
  else if(vjump>178)periodo=10;//valor para 10 horas
  else if(vjump>127)periodo=8;//valor para 8 horas
  else if(vjump>76)periodo=6;//valor para 6 horas
  else if(vjump>25)periodo=4;//valor para 4 horas
  else periodo=2;//valor para 2 horas
  vjump=periodo;
  }
/*******************************************************************************
               TESTA INTERRUPTOR 'START'
*******************************************************************************/
void Testa_SW(){
   if(!SW && !f_t1){
       f_t1=1;//houve uma programao
       out=SAIDA_DESLIGADA;//
       Le_Jumper();//le a programao do periodo escolhido pelo usurio
       hora=0;//zera contadores
       min=0;
       seg=0;
       f_prog=0;//houve programao
       n_piscada=1;
       f_disp=1; // liga a sada da lampada
      }
   if(f_prog)Pisca_Led();//se perdeu a programao ir piscar rpido
   }
/*******************************************************************************
               GERA PISCADA INFORMATIVAS DO PERIODO PROGRAMADO
*******************************************************************************/
void Info_led(){

   if(!n_piscada){
     t_pisc--;
     if(t_pisc==0) {n_piscada=periodo; t_pisc=PAUSE_PISCA;}
   }
   if(n_piscada) {
    n_piscada--;//decrementa
    LED=LED_LIGADO;//acende led prog
    Delay_Pisca_ON();
    LED=LED_DESLIGADO;//apaga led prog
   }
}
/*******************************************************************************
          COMPARA REGISTRADORES DE HORAS/MIN COM PERIODO PROGRAMADO
*******************************************************************************/
void Compara_Tempo(){

  if(f_disp && !f_prog){
    out=SAIDA_LIGADA;//liga saida
    if((hora==periodo) && (min==0) ){
    out=SAIDA_DESLIGADA;
    f_disp=0;
    }
  }
  else out=SAIDA_DESLIGADA;
}
/*******************************************************************************
               ROTINA MAIN
*******************************************************************************/

void main() {

   setup_adc_ports(sAN2|VSS_VDD);//pino 5 como entrada AD
   setup_adc(ADC_CLOCK_INTERNAL);// ajusta o clock do adc para o oscilador dedicado
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_32);//tmr0 : 32
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);//desliga comparador interno
   setup_vref(FALSE);
   TRISIO=0B111100;//ajusta sadas e entradas
   TMR0=0;//zera timer
   t0if=0;//apaga flag de over flow do tmr0
   out=SAIDA_DESLIGADA;//desliga sada
   LED=LED_DESLIGADO;//apaga led
   f_prog=1;//condio inicial para indicar queda de tenso
/*******************************************************************************
              LAO DE REPETIO
*******************************************************************************/
   for(;;){
        while( !t0if) Testa_SW();//aguarda passar um segundo e testa o boto
        t0if=0;//apaga flag de overflow do timer 0
        Atualiza_Cont_Tempo();//incrementa contadores de tempo (segundos, minutos, horas)
        Compara_Tempo();//compara tempo atual com o periodo programado pelo jumper.
        Info_led();//gera piscadas informativas  para indicar o tempo programado
        f_t1=0;//habilita nova leitura em SW

   }//for(;;)

}//main()
